pub trait SystemFunction<P: SystemParam> {
// Required method
fn run_function(&self, parameter: SystemParamItem<'_, P>);
}Expand description
A trait implemented for all functions that can be used as a System
pub trait SystemFunction<P: SystemParam> {
// Required method
fn run_function(&self, parameter: SystemParamItem<'_, P>);
}A trait implemented for all functions that can be used as a System